import os
from django.apps import AppConfig

from .classes.auto_migration_mixin import AutoMigrationMixin
from .classes.auto_urlpatterns_mixin import AutoUrlPatternsMixin


class ValarConfig(AutoMigrationMixin,AutoUrlPatternsMixin,AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = __package__

    def ready(self):
        if os.environ.get('RUN_MAIN') == 'true':
            getattr(super(), 'set_url', None)()
            getattr(super(), 'auto_migrate', None)()
            print(__package__)








class Counter:
    def __init__(self, array_or_int):
        self.length = array_or_int if isinstance(array_or_int,int) else len(array_or_int)
        self.index = 0

    def tick(self):
        self.index += 1
        percentage = round(self.index * 100 / self.length) if self.length else 0
        return {"index": self.index, "length": self.length, "percentage": percentage}


class AutoMigrationMixin:
    name = None  # 子类必须提供
    def auto_migrate(self):
        from django.core.management import call_command
        app = self.name.replace('src.','')
        call_command('makemigrations', app, interactive=False, verbosity=0)
        call_command('migrate', app, interactive=False, verbosity=0)




# Generated by Django 4.2.23 on 2025-07-15 18:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Meta',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('saved', models.BooleanField(default=False)),
                ('db', models.CharField(max_length=100, null=True, verbose_name='数据库')),
                ('entity', models.CharField(max_length=100, null=True, verbose_name='数据源')),
                ('name', models.CharField(max_length=50, null=True, verbose_name='实体别名')),
            ],
            options={
                'verbose_name': '数据实体',
                'unique_together': {('db', 'entity')},
            },
        ),
        migrations.CreateModel(
            name='MetaFieldTool',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('saved', models.BooleanField(default=False)),
                ('pid', models.IntegerField(default=0, verbose_name='父节点')),
                ('isLeaf', models.BooleanField(default=False, verbose_name='叶子节点')),
                ('icon', models.CharField(max_length=255, null=True, verbose_name='图标')),
                ('name', models.CharField(max_length=255, null=True, verbose_name='名称')),
                ('code', models.CharField(max_length=100, null=True, unique=True, verbose_name='代码')),
            ],
            options={
                'verbose_name': '元数据字段工具',
            },
        ),
        migrations.CreateModel(
            name='MetaView',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('saved', models.BooleanField(default=False)),
                ('code', models.CharField(default='default ', max_length=50, verbose_name='类视图')),
                ('name', models.CharField(max_length=50, null=True, verbose_name='视图名称')),
                ('form_width', models.IntegerField(default=0, verbose_name='表单宽度')),
                ('form_height', models.IntegerField(default=0, verbose_name='表单高度')),
                ('table_width', models.IntegerField(default=0, verbose_name='表格宽度')),
                ('table_height', models.IntegerField(default=0, verbose_name='表格高度')),
                ('enable', models.BooleanField(default=True, verbose_name='是否启用')),
                ('show_header', models.BooleanField(default=True, verbose_name='展示头部')),
                ('allow_search', models.BooleanField(default=True, verbose_name='检索功能')),
                ('allow_sort', models.BooleanField(default=True, verbose_name='移动功能')),
                ('allow_order', models.BooleanField(default=True, verbose_name='排序功能')),
                ('allow_insert', models.BooleanField(default=True, verbose_name='新增功能')),
                ('allow_edit', models.BooleanField(default=True, verbose_name='编辑功能')),
                ('allow_remove', models.BooleanField(default=True, verbose_name='删除功能')),
                ('allow_download', models.BooleanField(default=True, verbose_name='下载功能')),
                ('allow_upload', models.BooleanField(default=True, verbose_name='上传功能')),
                ('meta', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='valar.meta', verbose_name='元数据')),
            ],
            options={
                'verbose_name': '数据视图',
                'unique_together': {('meta', 'code')},
            },
        ),
        migrations.CreateModel(
            name='MetaFieldDomain',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('saved', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=255, null=True, unique=True, verbose_name='名称')),
                ('align', models.CharField(max_length=10, null=True, verbose_name='对齐方式')),
                ('default', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='valar.metafieldtool', verbose_name='默认工具')),
                ('search', models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='valar.metafieldtool', verbose_name='搜索工具')),
                ('tools', models.ManyToManyField(to='valar.metafieldtool', verbose_name='工具集')),
            ],
            options={
                'verbose_name': '元数据字段类型',
            },
        ),
        migrations.CreateModel(
            name='MetaField',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('sort', models.BigIntegerField(null=True, verbose_name='序号')),
                ('create_time', models.DateTimeField(auto_now_add=True, null=True, verbose_name='创建时间')),
                ('modify_time', models.DateTimeField(auto_now=True, null=True, verbose_name='修改时间')),
                ('saved', models.BooleanField(default=False)),
                ('prop', models.CharField(max_length=100, verbose_name='字段名称')),
                ('label', models.CharField(max_length=100, verbose_name='字段标签')),
                ('name', models.CharField(max_length=100, verbose_name='字段别名')),
                ('domain', models.CharField(max_length=100, verbose_name='字段类型')),
                ('tool', models.CharField(default='default', max_length=100, verbose_name='工具组件')),
                ('refer', models.JSONField(default=dict, verbose_name='索引')),
                ('format', models.JSONField(default=dict, verbose_name='格式')),
                ('not_null', models.BooleanField(default=False, verbose_name='不为空')),
                ('allow_edit', models.BooleanField(default=True, verbose_name='可编辑')),
                ('allow_order', models.BooleanField(default=True, verbose_name='可排序')),
                ('allow_search', models.BooleanField(default=True, verbose_name='可搜索')),
                ('allow_download', models.BooleanField(default=True, verbose_name='可下载')),
                ('allow_upload', models.BooleanField(default=False, verbose_name='可上传')),
                ('allow_update', models.BooleanField(default=False, verbose_name='可更新')),
                ('unit', models.CharField(max_length=55, null=True, verbose_name='单位符')),
                ('column_width', models.FloatField(default=0, verbose_name='表头宽度')),
                ('align', models.CharField(default='left', max_length=55, verbose_name='对齐方式')),
                ('fixed', models.CharField(max_length=100, null=True, verbose_name='固定位置')),
                ('header_color', models.CharField(max_length=55, null=True, verbose_name='表头颜色')),
                ('cell_color', models.CharField(max_length=55, null=True, verbose_name='单元颜色')),
                ('edit_on_table', models.BooleanField(default=True, verbose_name='表格编辑')),
                ('hide_on_table', models.BooleanField(default=False, verbose_name='表内隐藏')),
                ('span', models.IntegerField(default=0, verbose_name='表单占位')),
                ('hide_on_form', models.BooleanField(default=False, verbose_name='表单隐藏')),
                ('hide_on_form_edit', models.BooleanField(default=False, verbose_name='编辑隐藏')),
                ('hide_on_form_insert', models.BooleanField(default=False, verbose_name='新增隐藏')),
                ('hide_on_form_branch', models.BooleanField(default=False, verbose_name='分支隐藏')),
                ('hide_on_form_leaf', models.BooleanField(default=False, verbose_name='叶子隐藏')),
                ('view', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='valar.metaview', verbose_name='数据视图')),
            ],
            options={
                'verbose_name': '视图字段',
            },
        ),
    ]




from .executer import execute_channel
from .mapping import ChannelMapping
from .sender import ValarSocketSender
from ..classes.valar_response import ValarResponse


async def handel_channel(request, handler):
    sender = ValarSocketSender(request)
    method = ChannelMapping().get_handler(handler)
    await execute_channel(method, sender)
    return ValarResponse(True)



import json
import time
from datetime import datetime

from asgiref.sync import async_to_sync
from channels.layers import get_channel_layer
from django.http import HttpRequest

from .consumer import VALAR_CHANNEL_GROUP
from ..classes.counter import Counter


class ValarSocketSender:
    def __init__(self, request: HttpRequest, data=None):
        body = json.loads(request.body)
        auth = request.headers.get('AUTH')
        self.client = request.headers.get('CLIENT')
        self.uid = request.session.get('UID')
        self.handlerKey = body.get('handlerKey')
        self.channelKey = body.get('channelKey', 'default')
        self.data = data or body.get('data')
        self.send = get_channel_layer().group_send
        self.start_time = time.time()
        if auth and not self.uid:
            raise Exception('Unauthorized!')

    @staticmethod
    def create_counter(length:int):
        return Counter(length)

    def __convert_body(self, emit, payload, status ,clients = None, users = None):
        return {
            'type': emit,
            'data': {
                'status': status,
                'handlerKey': self.handlerKey,
                'channelKey': self.channelKey,
                'payload': payload,
                'timestamp': datetime.now().timestamp()
            },
            'clients': clients or [],
            'users': users or [],
        }


    def to_users(self, payload,  users, status='proceed'):
        body = self.__convert_body(emit='user.emit', payload=payload, status=status, users=users)
        async_to_sync(self.send)(VALAR_CHANNEL_GROUP, body)

    def to_clients(self,payload, clients, status='proceed', wait=False):
        current_time = time.time()
        time_span = current_time - self.start_time
        if (wait and time_span > 1 and status == 'proceed') or not wait:
            body = self.__convert_body(emit='client.emit', payload=payload, status=status, clients=clients)
            async_to_sync(self.send)(VALAR_CHANNEL_GROUP, body)
            self.start_time = current_time

    def broadcast(self, payload, status):
        body = self.__convert_body(emit='broadcast.emit', payload=payload, status=status)
        async_to_sync(self.send)(VALAR_CHANNEL_GROUP, body)

    def register(self):
        body = self.__convert_body(emit='register.emit',  payload=None, status=None,clients=[self.client], users=[self.uid])
        async_to_sync(self.send)(VALAR_CHANNEL_GROUP, body)



from channels.generic.websocket import AsyncJsonWebsocketConsumer

VALAR_CHANNEL_GROUP = 'VALAR'

class ValarConsumer(AsyncJsonWebsocketConsumer):

    def __init__(self):
        self.client = None
        self.uid = None
        super().__init__()

    async def connect(self):
        params = self.scope['url_route']['kwargs']
        self.client = params.get('client')
        await self.channel_layer.group_add(VALAR_CHANNEL_GROUP, self.channel_name)
        await self.accept()

    async def disconnect(self, code):
        await self.channel_layer.group_discard(VALAR_CHANNEL_GROUP, self.channel_name)
        await self.close(code)

    async def receive_json(self, data, *args, **kwargs):
        pass

    async def user_emit(self, event):
        users: list = event.get('users',[])
        data = event.get('data',{})
        if self.uid in users:
            await self.send_json(data)

    async def client_emit(self, event):
        clients: list = event.get('clients',[])
        data = event.get('data',{})
        if self.client in clients:
            await self.send_json(data)

    async def broadcast_emit(self, event):
        data = event.get('data',{})
        await self.send_json(data)

    async def register_emit(self, event):
        users: list = event.get('users', [])
        clients: list = event.get('clients',[])
        if self.client in clients:
            self.uid = users[0]





import importlib

from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

from ..classes.singleton_meta import SingletonMeta

class ChannelMapping(metaclass=SingletonMeta):
    def __init__(self):
        mapping = settings.HANDLER_MAPPING
        root, name = mapping.rsplit('.', 1)
        module = importlib.import_module(root)
        if hasattr(module, name):
            self.mapping: dict = getattr(module, name)
        else:
            raise ImproperlyConfigured("%r has no attribute %r" % (root, name))

    def get_handler(self, handler):
        method = self.mapping.get(handler)
        if method is None:
            raise ImproperlyConfigured("Cannot find handler - %r" % handler)
        return method

